@@ -1,22 +1,22 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#if !defined(_NEPIMPL)
-#define _NEPIMPL
+#if !defined(SLEPCNEPIMPL_H)
+#define SLEPCNEPIMPL_H
 
 #include <slepcnep.h>
 #include <slepc/private/slepcimpl.h>
 
-PETSC_EXTERN PetscBool NEPRegisterAllCalled;
-PETSC_EXTERN PetscErrorCode NEPRegisterAll(void);
-PETSC_EXTERN PetscLogEvent NEP_SetUp,NEP_Solve,NEP_Refine,NEP_FunctionEval,NEP_JacobianEval,NEP_DerivativesEval;
+SLEPC_EXTERN PetscBool NEPRegisterAllCalled;
+SLEPC_EXTERN PetscErrorCode NEPRegisterAll(void);
+SLEPC_EXTERN PetscLogEvent NEP_SetUp,NEP_Solve,NEP_Refine,NEP_FunctionEval,NEP_JacobianEval,NEP_Resolvent;
 
 typedef struct _NEPOps *NEPOps;
 
@@ -45,11 +45,9 @@ typedef enum { NEP_STATE_INITIAL,
      How the problem function T(lambda) has been defined by the user
      - Callback: one callback to build the function matrix, another one for the Jacobian
      - Split: in split form sum_j(A_j*f_j(lambda))
-     - Derivatives: a single callback for all the derivatives (including the 0th derivative)
 */
 typedef enum { NEP_USER_INTERFACE_CALLBACK=1,
-               NEP_USER_INTERFACE_SPLIT,
-               NEP_USER_INTERFACE_DERIVATIVES } NEPUserInterface;
+               NEP_USER_INTERFACE_SPLIT } NEPUserInterface;
 
 /*
    Defines the NEP data structure.
@@ -74,14 +72,13 @@ struct _p_NEP {
   PetscInt       rits;             /* number of iterations of the refinement method */
   NEPRefineScheme scheme;          /* scheme for solving linear systems within refinement */
   PetscBool      trackall;         /* whether all the residuals must be computed */
+  PetscBool      twosided;         /* whether to compute left eigenvectors (two-sided solver) */
 
   /*-------------- User-provided functions and contexts -----------------*/
   PetscErrorCode (*computefunction)(NEP,PetscScalar,Mat,Mat,void*);
   PetscErrorCode (*computejacobian)(NEP,PetscScalar,Mat,void*);
   void           *functionctx;
   void           *jacobianctx;
-  PetscErrorCode (*computederivatives)(NEP,PetscScalar,PetscInt,Mat,void*);
-  void           *derivativesctx;
   PetscErrorCode (*converged)(NEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
   PetscErrorCode (*convergeduser)(NEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
   PetscErrorCode (*convergeddestroy)(void*);
@@ -98,12 +95,12 @@ struct _p_NEP {
   /*----------------- Child objects and working data -------------------*/
   DS             ds;               /* direct solver object */
   BV             V;                /* set of basis vectors and computed eigenvectors */
+  BV             W;                /* left basis vectors (if left eigenvectors requested) */
   RG             rg;               /* optional region for filtering */
   SlepcSC        sc;               /* sorting criterion data */
   Mat            function;         /* function matrix */
   Mat            function_pre;     /* function matrix (preconditioner) */
   Mat            jacobian;         /* Jacobian matrix */
-  Mat            derivatives;      /* derivatives matrix */
   Mat            *A;               /* matrix coefficients of split form */
   FN             *f;               /* matrix functions of split form */
   PetscInt       nt;               /* number of terms in split form */
@@ -125,6 +122,9 @@ struct _p_NEP {
   PetscInt       n,nloc;           /* problem dimensions (global, local) */
   PetscReal      *nrma;            /* computed matrix norms */
   NEPUserInterface fui;            /* how the user has defined the nonlinear operator */
+  PetscBool      useds;            /* whether the solver uses the DS object or not */
+  PetscBool      hasts;            /* whether the solver has two-sided variant */
+  Mat            resolvent;        /* shell matrix to be used in NEPApplyResolvent */
   NEPConvergedReason reason;
 };
 
@@ -143,37 +143,32 @@ struct _p_NEP {
 
 #define NEPCheckProblem(h,arg) \
   do { \
-    if (!(h->fui)) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"The nonlinear eigenproblem has not been specified yet. Parameter #%d",arg); \
+    if (!((h)->fui)) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"The nonlinear eigenproblem has not been specified yet. Parameter #%d",arg); \
   } while (0)
 
 #define NEPCheckCallback(h,arg) \
   do { \
-    if (h->fui!=NEP_USER_INTERFACE_CALLBACK) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"This operation requires the nonlinear eigenproblem specified with callbacks. Parameter #%d",arg); \
+    if ((h)->fui!=NEP_USER_INTERFACE_CALLBACK) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"This operation requires the nonlinear eigenproblem specified with callbacks. Parameter #%d",arg); \
   } while (0)
 
 #define NEPCheckSplit(h,arg) \
   do { \
-    if (h->fui!=NEP_USER_INTERFACE_SPLIT) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"This operation requires the nonlinear eigenproblem in split form. Parameter #%d",arg); \
-  } while (0)
-
-#define NEPCheckDerivatives(h,arg) \
-  do { \
-    if (h->fui!=NEP_USER_INTERFACE_DERIVATIVES) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"This operation requires the nonlinear eigenproblem specified with derivatives callback. Parameter #%d",arg); \
+    if ((h)->fui!=NEP_USER_INTERFACE_SPLIT) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"This operation requires the nonlinear eigenproblem in split form. Parameter #%d",arg); \
   } while (0)
 
 #define NEPCheckSolved(h,arg) \
   do { \
-    if (h->state<NEP_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"Must call NEPSolve() first: Parameter #%d",arg); \
+    if ((h)->state<NEP_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"Must call NEPSolve() first: Parameter #%d",arg); \
   } while (0)
 
 #endif
 
-PETSC_INTERN PetscErrorCode NEPSetDimensions_Default(NEP,PetscInt,PetscInt*,PetscInt*);
-PETSC_INTERN PetscErrorCode NEPComputeVectors(NEP);
-PETSC_INTERN PetscErrorCode NEPReset_Problem(NEP);
-PETSC_INTERN PetscErrorCode NEPGetDefaultShift(NEP,PetscScalar*);
-PETSC_INTERN PetscErrorCode NEPComputeVectors_Schur(NEP);
-PETSC_INTERN PetscErrorCode NEPComputeResidualNorm_Private(NEP,PetscScalar,Vec,Vec*,PetscReal*);
-PETSC_INTERN PetscErrorCode NEPNewtonRefinementSimple(NEP,PetscInt*,PetscReal,PetscInt);
+SLEPC_INTERN PetscErrorCode NEPSetDimensions_Default(NEP,PetscInt,PetscInt*,PetscInt*);
+SLEPC_INTERN PetscErrorCode NEPComputeVectors(NEP);
+SLEPC_INTERN PetscErrorCode NEPReset_Problem(NEP);
+SLEPC_INTERN PetscErrorCode NEPGetDefaultShift(NEP,PetscScalar*);
+SLEPC_INTERN PetscErrorCode NEPComputeVectors_Schur(NEP);
+SLEPC_INTERN PetscErrorCode NEPComputeResidualNorm_Private(NEP,PetscBool,PetscScalar,Vec,Vec*,PetscReal*);
+SLEPC_INTERN PetscErrorCode NEPNewtonRefinementSimple(NEP,PetscInt*,PetscReal,PetscInt);
 
 #endif
