@@ -1,22 +1,22 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#if !defined(_PEPIMPL)
-#define _PEPIMPL
+#if !defined(SLEPCPEPIMPL_H)
+#define SLEPCPEPIMPL_H
 
 #include <slepcpep.h>
 #include <slepc/private/slepcimpl.h>
 
-PETSC_EXTERN PetscBool PEPRegisterAllCalled;
-PETSC_EXTERN PetscErrorCode PEPRegisterAll(void);
-PETSC_EXTERN PetscLogEvent PEP_SetUp,PEP_Solve,PEP_Refine;
+SLEPC_EXTERN PetscBool PEPRegisterAllCalled;
+SLEPC_EXTERN PetscErrorCode PEPRegisterAll(void);
+SLEPC_EXTERN PetscLogEvent PEP_SetUp,PEP_Solve,PEP_Refine;
 
 typedef struct _PEPOps *PEPOps;
 
@@ -60,6 +60,7 @@ struct _p_PEP {
   PEPConv        conv;             /* convergence test */
   PEPStop        stop;             /* stopping test */
   PEPWhich       which;            /* which part of the spectrum to be sought */
+  PetscReal      inta,intb;        /* interval [a,b] for spectrum slicing */
   PEPBasis       basis;            /* polynomial basis used to represent the problem */
   PEPProblemType problem_type;     /* which kind of problem to be solved */
   PEPScale       scale;            /* scaling strategy to be used */
@@ -132,26 +133,28 @@ struct _p_PEP {
 
 #define PEPCheckSolved(h,arg) \
   do { \
-    if (h->state<PEP_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"Must call PEPSolve() first: Parameter #%d",arg); \
+    if ((h)->state<PEP_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"Must call PEPSolve() first: Parameter #%d",arg); \
   } while (0)
 
 #endif
 
-PETSC_INTERN PetscErrorCode PEPSetDimensions_Default(PEP,PetscInt,PetscInt*,PetscInt*);
-PETSC_INTERN PetscErrorCode PEPExtractVectors(PEP);
-PETSC_INTERN PetscErrorCode PEPBackTransform_Default(PEP);
-PETSC_INTERN PetscErrorCode PEPComputeVectors(PEP);
-PETSC_INTERN PetscErrorCode PEPComputeVectors_Default(PEP);
-PETSC_INTERN PetscErrorCode PEPComputeVectors_Indefinite(PEP);
-PETSC_INTERN PetscErrorCode PEPComputeResidualNorm_Private(PEP,PetscScalar,PetscScalar,Vec,Vec,Vec*,PetscReal*);
-PETSC_INTERN PetscErrorCode PEPKrylovConvergence(PEP,PetscBool,PetscInt,PetscInt,PetscReal,PetscInt*);
-PETSC_INTERN PetscErrorCode PEPComputeScaleFactor(PEP);
-PETSC_INTERN PetscErrorCode PEPBuildDiagonalScaling(PEP);
-PETSC_INTERN PetscErrorCode PEPBasisCoefficients(PEP,PetscReal*);
-PETSC_INTERN PetscErrorCode PEPEvaluateBasis(PEP,PetscScalar,PetscScalar,PetscScalar*,PetscScalar*);
-PETSC_INTERN PetscErrorCode PEPNewtonRefinement_TOAR(PEP,PetscScalar,PetscInt*,PetscReal*,PetscInt,PetscScalar*,PetscInt);
-PETSC_INTERN PetscErrorCode PEPNewtonRefinementSimple(PEP,PetscInt*,PetscReal,PetscInt);
-PETSC_INTERN PetscErrorCode PEPSetDefaultST(PEP);
-PETSC_INTERN PetscErrorCode PEPSetDefaultST_Transform(PEP);
+SLEPC_INTERN PetscErrorCode PEPSetDimensions_Default(PEP,PetscInt,PetscInt*,PetscInt*);
+SLEPC_INTERN PetscErrorCode PEPExtractVectors(PEP);
+SLEPC_INTERN PetscErrorCode PEPBackTransform_Default(PEP);
+SLEPC_INTERN PetscErrorCode PEPComputeVectors(PEP);
+SLEPC_INTERN PetscErrorCode PEPComputeVectors_Default(PEP);
+SLEPC_INTERN PetscErrorCode PEPComputeVectors_Indefinite(PEP);
+SLEPC_INTERN PetscErrorCode PEPComputeResidualNorm_Private(PEP,PetscScalar,PetscScalar,Vec,Vec,Vec*,PetscReal*);
+SLEPC_INTERN PetscErrorCode PEPKrylovConvergence(PEP,PetscBool,PetscInt,PetscInt,PetscReal,PetscInt*);
+SLEPC_INTERN PetscErrorCode PEPComputeScaleFactor(PEP);
+SLEPC_INTERN PetscErrorCode PEPBuildDiagonalScaling(PEP);
+SLEPC_INTERN PetscErrorCode PEPBasisCoefficients(PEP,PetscReal*);
+SLEPC_INTERN PetscErrorCode PEPEvaluateBasis(PEP,PetscScalar,PetscScalar,PetscScalar*,PetscScalar*);
+SLEPC_INTERN PetscErrorCode PEPEvaluateBasisDerivative(PEP,PetscScalar,PetscScalar,PetscScalar*,PetscScalar*);
+SLEPC_INTERN PetscErrorCode PEPEvaluateBasisMat(PEP,PetscInt,PetscScalar*,PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscInt);
+SLEPC_INTERN PetscErrorCode PEPNewtonRefinement_TOAR(PEP,PetscScalar,PetscInt*,PetscReal*,PetscInt,PetscScalar*,PetscInt);
+SLEPC_INTERN PetscErrorCode PEPNewtonRefinementSimple(PEP,PetscInt*,PetscReal,PetscInt);
+SLEPC_INTERN PetscErrorCode PEPSetDefaultST(PEP);
+SLEPC_INTERN PetscErrorCode PEPSetDefaultST_Transform(PEP);
 
 #endif
