@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,7 +21,7 @@ PetscLogEvent     LME_SetUp = 0,LME_Solve = 0,LME_ComputeError = 0;
 /*@C
    LMEView - Prints the LME data structure.
 
-   Collective on LME
+   Collective on lme
 
    Input Parameters:
 +  lme - the linear matrix equation solver context
@@ -42,14 +42,11 @@ PetscLogEvent     LME_SetUp = 0,LME_Solve = 0,LME_ComputeError = 0;
    PetscViewerASCIIOpen() - output to a specified file.
 
    Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
 @*/
 PetscErrorCode LMEView(LME lme,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      isascii;
-  PetscInt       tabs;
   const char     *eqname[] = {
                    "continuous-time Lyapunov",
                    "continuous-time Sylvester",
@@ -61,14 +58,14 @@ PetscErrorCode LMEView(LME lme,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(lme,LME_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)lme));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)lme),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(lme,1,viewer,2);
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)lme)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)lme,viewer);CHKERRQ(ierr);
     if (lme->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -79,7 +76,6 @@ PetscErrorCode LMEView(LME lme,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",lme->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",lme->max_it);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",(double)lme->tol);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else {
     if (lme->ops->view) {
       ierr = (*lme->ops->view)(lme,viewer);CHKERRQ(ierr);
@@ -93,11 +89,34 @@ PetscErrorCode LMEView(LME lme,PetscViewer viewer)
 }
 
 /*@C
-   LMEReasonView - Displays the reason an LME solve converged or diverged.
+   LMEViewFromOptions - View from options
 
    Collective on LME
 
-   Parameter:
+   Input Parameters:
++  lme  - the linear matrix equation context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: LMEView(), LMECreate()
+@*/
+PetscErrorCode LMEViewFromOptions(LME lme,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(lme,LME_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)lme,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+/*@C
+   LMEReasonView - Displays the reason an LME solve converged or diverged.
+
+   Collective on lme
+
+   Input Parameters:
 +  lme - the linear matrix equation context
 -  viewer - the viewer to display the reason
 
@@ -131,9 +150,9 @@ PetscErrorCode LMEReasonView(LME lme,PetscViewer viewer)
    LMEReasonViewFromOptions - Processes command line options to determine if/how
    the LME converged reason is to be viewed.
 
-   Collective on LME
+   Collective on lme
 
-   Input Parameters:
+   Input Parameter:
 .  lme - the linear matrix equation context
 
    Level: developer
@@ -149,7 +168,7 @@ PetscErrorCode LMEReasonViewFromOptions(LME lme)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)lme),((PetscObject)lme)->prefix,"-lme_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)lme),((PetscObject)lme)->options,((PetscObject)lme)->prefix,"-lme_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = LMEReasonView(lme,viewer);CHKERRQ(ierr);
@@ -163,7 +182,7 @@ PetscErrorCode LMEReasonViewFromOptions(LME lme)
 /*@
    LMECreate - Creates the default LME context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -220,7 +239,7 @@ PetscErrorCode LMECreate(MPI_Comm comm,LME *outlme)
 /*@C
    LMESetType - Selects the particular solver to be used in the LME object.
 
-   Logically Collective on LME
+   Logically Collective on lme
 
    Input Parameters:
 +  lme  - the linear matrix equation context
@@ -325,6 +344,7 @@ PetscErrorCode LMERegister(const char *name,PetscErrorCode (*function)(LME))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = LMEInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&LMEList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -333,7 +353,7 @@ PetscErrorCode LMERegister(const char *name,PetscErrorCode (*function)(LME))
    LMEReset - Resets the LME context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on LME
+   Collective on lme
 
    Input Parameter:
 .  lme - linear matrix equation context obtained from LMECreate()
@@ -366,7 +386,7 @@ PetscErrorCode LMEReset(LME lme)
 /*@
    LMEDestroy - Destroys the LME context.
 
-   Collective on LME
+   Collective on lme
 
    Input Parameter:
 .  lme - linear matrix equation context obtained from LMECreate()
@@ -393,7 +413,7 @@ PetscErrorCode LMEDestroy(LME *lme)
 /*@
    LMESetBV - Associates a basis vectors object to the linear matrix equation solver.
 
-   Collective on LME
+   Collective on lme
 
    Input Parameters:
 +  lme - linear matrix equation context obtained from LMECreate()
@@ -447,7 +467,9 @@ PetscErrorCode LMEGetBV(LME lme,BV *bv)
   PetscValidPointer(bv,2);
   if (!lme->V) {
     ierr = BVCreate(PetscObjectComm((PetscObject)lme),&lme->V);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)lme->V,(PetscObject)lme,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)lme,(PetscObject)lme->V);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)lme->V,((PetscObject)lme)->options);CHKERRQ(ierr);
   }
   *bv = lme->V;
   PetscFunctionReturn(0);
