@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -16,12 +16,12 @@
 PetscFunctionList NEPList = 0;
 PetscBool         NEPRegisterAllCalled = PETSC_FALSE;
 PetscClassId      NEP_CLASSID = 0;
-PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_DerivativesEval = 0;
+PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_Resolvent = 0;
 
 /*@
    NEPCreate - Creates the default NEP context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -61,13 +61,12 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->rits            = PETSC_DEFAULT;
   nep->scheme          = (NEPRefineScheme)0;
   nep->trackall        = PETSC_FALSE;
+  nep->twosided        = PETSC_FALSE;
 
   nep->computefunction = NULL;
   nep->computejacobian = NULL;
   nep->functionctx     = NULL;
   nep->jacobianctx     = NULL;
-  nep->computederivatives = NULL;
-  nep->derivativesctx  = NULL;
   nep->converged       = NEPConvergedRelative;
   nep->convergeduser   = NULL;
   nep->convergeddestroy= NULL;
@@ -80,11 +79,11 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
 
   nep->ds              = NULL;
   nep->V               = NULL;
+  nep->W               = NULL;
   nep->rg              = NULL;
   nep->function        = NULL;
   nep->function_pre    = NULL;
   nep->jacobian        = NULL;
-  nep->derivatives     = NULL;
   nep->A               = NULL;
   nep->f               = NULL;
   nep->nt              = 0;
@@ -105,6 +104,9 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->nloc            = 0;
   nep->nrma            = NULL;
   nep->fui             = (NEPUserInterface)0;
+  nep->useds           = PETSC_FALSE;
+  nep->hasts           = PETSC_FALSE;
+  nep->resolvent       = NULL;
   nep->reason          = NEP_CONVERGED_ITERATING;
 
   ierr = PetscNewLog(nep,&nep->sc);CHKERRQ(ierr);
@@ -115,7 +117,7 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
 /*@C
    NEPSetType - Selects the particular solver to be used in the NEP object.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep      - the nonlinear eigensolver context
@@ -219,6 +221,7 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = NEPInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&NEPList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -236,7 +239,6 @@ PetscErrorCode NEPReset_Problem(NEP nep)
   ierr = MatDestroy(&nep->function);CHKERRQ(ierr);
   ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
   ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->derivatives);CHKERRQ(ierr);
   if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
     ierr = MatDestroyMatrices(nep->nt,&nep->A);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
@@ -252,7 +254,7 @@ PetscErrorCode NEPReset_Problem(NEP nep)
    NEPReset - Resets the NEP context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameter:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -272,7 +274,9 @@ PetscErrorCode NEPReset(NEP nep)
   if (nep->refineksp) { ierr = KSPReset(nep->refineksp);CHKERRQ(ierr); }
   ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
   ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
+  ierr = BVDestroy(&nep->W);CHKERRQ(ierr);
   ierr = VecDestroyVecs(nep->nwork,&nep->work);CHKERRQ(ierr);
+  ierr = MatDestroy(&nep->resolvent);CHKERRQ(ierr);
   nep->nwork = 0;
   nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -281,7 +285,7 @@ PetscErrorCode NEPReset(NEP nep)
 /*@
    NEPDestroy - Destroys the NEP context.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameter:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -321,7 +325,7 @@ PetscErrorCode NEPDestroy(NEP *nep)
 /*@
    NEPSetBV - Associates a basis vectors object to the nonlinear eigensolver.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -375,7 +379,9 @@ PetscErrorCode NEPGetBV(NEP nep,BV *bv)
   PetscValidPointer(bv,2);
   if (!nep->V) {
     ierr = BVCreate(PetscObjectComm((PetscObject)nep),&nep->V);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->V,(PetscObject)nep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)nep->V,((PetscObject)nep)->options);CHKERRQ(ierr);
   }
   *bv = nep->V;
   PetscFunctionReturn(0);
@@ -384,7 +390,7 @@ PetscErrorCode NEPGetBV(NEP nep,BV *bv)
 /*@
    NEPSetRG - Associates a region object to the nonlinear eigensolver.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -438,7 +444,9 @@ PetscErrorCode NEPGetRG(NEP nep,RG *rg)
   PetscValidPointer(rg,2);
   if (!nep->rg) {
     ierr = RGCreate(PetscObjectComm((PetscObject)nep),&nep->rg);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->rg,(PetscObject)nep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)nep->rg,((PetscObject)nep)->options);CHKERRQ(ierr);
   }
   *rg = nep->rg;
   PetscFunctionReturn(0);
@@ -447,7 +455,7 @@ PetscErrorCode NEPGetRG(NEP nep,RG *rg)
 /*@
    NEPSetDS - Associates a direct solver object to the nonlinear eigensolver.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -501,7 +509,9 @@ PetscErrorCode NEPGetDS(NEP nep,DS *ds)
   PetscValidPointer(ds,2);
   if (!nep->ds) {
     ierr = DSCreate(PetscObjectComm((PetscObject)nep),&nep->ds);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->ds,(PetscObject)nep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)nep->ds,((PetscObject)nep)->options);CHKERRQ(ierr);
   }
   *ds = nep->ds;
   PetscFunctionReturn(0);
@@ -539,7 +549,9 @@ PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
       ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
     }
     ierr = KSPCreate((nep->npart==1)?PetscObjectComm((PetscObject)nep):PetscSubcommChild(nep->refinesubc),&nep->refineksp);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->refineksp,(PetscObject)nep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)nep->refineksp,((PetscObject)nep)->options);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix);CHKERRQ(ierr);
     ierr = KSPAppendOptionsPrefix(*ksp,"nep_refine_");CHKERRQ(ierr);
     ierr = KSPSetTolerances(nep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
@@ -551,7 +563,7 @@ PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
 /*@
    NEPSetTarget - Sets the value of the target.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep    - eigensolver context
@@ -612,7 +624,7 @@ PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
    NEPSetFunction - Sets the function to compute the nonlinear Function T(lambda)
    as well as the location to store the matrix.
 
-   Logically Collective on NEP and Mat
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - the NEP context
@@ -703,7 +715,7 @@ PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,P
    NEPSetJacobian - Sets the function to compute Jacobian T'(lambda) as well
    as the location to store the matrix.
 
-   Logically Collective on NEP and Mat
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - the NEP context
@@ -783,7 +795,7 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
    NEPSetSplitOperator - Sets the operator of the nonlinear eigenvalue problem
    in split form.
 
-   Collective on NEP, Mat and FN
+   Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -823,24 +835,24 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructur
   PetscCheckSameComm(nep,1,*A,3);
   PetscValidPointer(f,4);
   PetscCheckSameComm(nep,1,*f,4);
+
+  for (i=0;i<n;i++) {
+    PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
+    PetscValidHeaderSpecific(f[i],FN_CLASSID,4);
+    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
+    ierr = PetscObjectReference((PetscObject)f[i]);CHKERRQ(ierr);
+  }
+
   if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
   else { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
 
   /* allocate space and copy matrices and functions */
   ierr = PetscMalloc1(n,&nep->A);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(Mat));CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
-    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
-    nep->A[i] = A[i];
-  }
+  for (i=0;i<n;i++) nep->A[i] = A[i];
   ierr = PetscMalloc1(n,&nep->f);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(FN));CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    PetscValidHeaderSpecific(f[i],FN_CLASSID,4);
-    ierr = PetscObjectReference((PetscObject)f[i]);CHKERRQ(ierr);
-    nep->f[i] = f[i];
-  }
+  for (i=0;i<n;i++) nep->f[i] = f[i];
   ierr = PetscCalloc1(n,&nep->nrma);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(PetscReal));CHKERRQ(ierr);
   nep->nt    = n;
@@ -906,75 +918,3 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
   PetscFunctionReturn(0);
 }
 
-/*@C
-   NEPSetDerivatives - Sets the function to compute the k-th derivative T^(k)(lambda)
-   for any value of k (including 0), as well as the location to store the matrix.
-
-   Logically Collective on NEP and Mat
-
-   Input Parameters:
-+  nep - the NEP context
-.  A   - the matrix to store the computed derivative
-.  der - routing to evaluate the k-th derivative (if NULL then NEP retains any
-         previously set value)
--  ctx - [optional] user-defined context for private data for the derivatives
-         evaluation routine (may be NULL) (if NULL then NEP retains any
-         previously set value)
-
-   Level: beginner
-
-.seealso: NEPSetFunction(), NEPGetDerivatives()
-@*/
-PetscErrorCode NEPSetDerivatives(NEP nep,Mat A,PetscErrorCode (*der)(NEP,PetscScalar,PetscInt,Mat,void*),void *ctx)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
-  if (A) PetscCheckSameComm(nep,1,A,2);
-
-  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
-  else { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
-
-  if (der) nep->computederivatives = der;
-  if (ctx) nep->derivativesctx     = ctx;
-  if (A) {
-    ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-    ierr = MatDestroy(&nep->derivatives);CHKERRQ(ierr);
-    nep->derivatives = A;
-  }
-  nep->fui   = NEP_USER_INTERFACE_DERIVATIVES;
-  nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
-}
-
-/*@C
-   NEPGetDerivatives - Returns the derivatives matrix and optionally the user
-   provided routine and context for evaluating the derivatives.
-
-   Not Collective, but Mat object will be parallel if NEP object is
-
-   Input Parameter:
-.  nep - the nonlinear eigensolver context
-
-   Output Parameters:
-+  A   - location to stash the derivatives matrix (or NULL)
-.  der - location to put derivatives function (or NULL)
--  ctx - location to stash derivatives context (or NULL)
-
-   Level: advanced
-
-.seealso: NEPSetDerivatives()
-@*/
-PetscErrorCode NEPGetDerivatives(NEP nep,Mat *A,PetscErrorCode (**der)(NEP,PetscScalar,PetscInt,Mat,void*),void **ctx)
-{
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  NEPCheckDerivatives(nep,1);
-  if (A)   *A   = nep->derivatives;
-  if (der) *der = nep->computederivatives;
-  if (ctx) *ctx = nep->derivativesctx;
-  PetscFunctionReturn(0);
-}
-
