@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -48,7 +48,7 @@ PetscErrorCode PEPSetDefaultST_Transform(PEP pep)
    PEPSetUp - Sets up all the internal data structures necessary for the
    execution of the PEP solver.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameter:
 .  pep   - solver context
@@ -105,6 +105,8 @@ PetscErrorCode PEPSetUp(PEP pep)
   if (!pep->problem_type) {
     ierr = PEPSetProblemType(pep,PEP_GENERAL);CHKERRQ(ierr);
   }
+  if (pep->nev > (pep->nmat-1)*pep->n) pep->nev = (pep->nmat-1)*pep->n;
+  if (pep->ncv > (pep->nmat-1)*pep->n) pep->ncv = (pep->nmat-1)*pep->n;
 
   /* check consistency of refinement options */
   if (pep->refine) {
@@ -255,7 +257,7 @@ PetscErrorCode PEPSetUp(PEP pep)
    PEPSetOperators - Sets the coefficient matrices associated with the polynomial
    eigenvalue problem.
 
-   Collective on PEP and Mat
+   Collective on pep
 
    Input Parameters:
 +  pep  - the eigenproblem solver context
@@ -275,7 +277,7 @@ PetscErrorCode PEPSetUp(PEP pep)
 PetscErrorCode PEPSetOperators(PEP pep,PetscInt nmat,Mat A[])
 {
   PetscErrorCode ierr;
-  PetscInt       i,n,m,m0=0;
+  PetscInt       i,n=0,m,m0=0;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
@@ -283,9 +285,17 @@ PetscErrorCode PEPSetOperators(PEP pep,PetscInt nmat,Mat A[])
   if (nmat <= 0) SETERRQ1(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Non-positive value of nmat: %D",nmat);
   if (nmat <= 2) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Cannot solve linear eigenproblems with PEP; use EPS instead");
   PetscValidPointer(A,3);
-  PetscValidHeaderSpecific(A[0],MAT_CLASSID,3);
 
-  ierr = MatGetSize(A[0],&m,&n);CHKERRQ(ierr);
+  for (i=0;i<nmat;i++) {
+    PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
+    PetscCheckSameComm(pep,1,A[i],3);
+    ierr = MatGetSize(A[i],&m,&n);CHKERRQ(ierr);
+    if (m!=n) SETERRQ1(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"A[%D] is a non-square matrix",i);
+    if (!i) m0 = m;
+    if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_INCOMP,"Dimensions of matrices do not match with each other");
+    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
+  }
+
   if (pep->state && n!=pep->n) { ierr = PEPReset(pep);CHKERRQ(ierr); }
   else if (pep->nmat) {
     ierr = MatDestroyMatrices(pep->nmat,&pep->A);CHKERRQ(ierr);
@@ -295,17 +305,10 @@ PetscErrorCode PEPSetOperators(PEP pep,PetscInt nmat,Mat A[])
 
   ierr = PetscMalloc1(nmat,&pep->A);CHKERRQ(ierr);
   ierr = PetscCalloc2(3*nmat,&pep->pbc,nmat,&pep->nrma);CHKERRQ(ierr);
-  for (i=0;i<nmat;i++) pep->pbc[i] = 1.0;  /* default to monomial basis */
   ierr = PetscLogObjectMemory((PetscObject)pep,nmat*sizeof(Mat)+4*nmat*sizeof(PetscReal));CHKERRQ(ierr);
   for (i=0;i<nmat;i++) {
-    PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
-    PetscCheckSameComm(pep,1,A[i],3);
-    ierr = MatGetSize(A[i],&m,&n);CHKERRQ(ierr);
-    if (m!=n) SETERRQ1(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"A[%D] is a non-square matrix",i);
-    if (!i) m0 = m;
-    if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_INCOMP,"Dimensions of matrices do not match with each other");
-    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
-    pep->A[i] = A[i];
+    pep->A[i]   = A[i];
+    pep->pbc[i] = 1.0;  /* default to monomial basis */
   }
   pep->nmat = nmat;
   pep->state = PEP_STATE_INITIAL;
@@ -357,7 +360,7 @@ PetscErrorCode PEPGetNumMatrices(PEP pep,PetscInt *nmat)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(nmat,2);
+  PetscValidIntPointer(nmat,2);
   *nmat = pep->nmat;
   PetscFunctionReturn(0);
 }
@@ -366,7 +369,7 @@ PetscErrorCode PEPGetNumMatrices(PEP pep,PetscInt *nmat)
    PEPSetInitialSpace - Specify a basis of vectors that constitute the initial
    space, that is, the subspace from which the solver starts to iterate.
 
-   Collective on PEP and Vec
+   Collective on pep
 
    Input Parameter:
 +  pep   - the polynomial eigensolver context
@@ -388,7 +391,7 @@ PetscErrorCode PEPGetNumMatrices(PEP pep,PetscInt *nmat)
 
    Level: intermediate
 @*/
-PetscErrorCode PEPSetInitialSpace(PEP pep,PetscInt n,Vec *is)
+PetscErrorCode PEPSetInitialSpace(PEP pep,PetscInt n,Vec is[])
 {
   PetscErrorCode ierr;
 
@@ -396,6 +399,10 @@ PetscErrorCode PEPSetInitialSpace(PEP pep,PetscInt n,Vec *is)
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(pep,n,2);
   if (n<0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  if (n>0) {
+    PetscValidPointer(is,3);
+    PetscValidHeaderSpecific(*is,VEC_CLASSID,3);
+  }
   ierr = SlepcBasisReference_Private(n,is,&pep->nini,&pep->IS);CHKERRQ(ierr);
   if (n>0) pep->state = PEP_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -412,8 +419,8 @@ PetscErrorCode PEPSetDimensions_Default(PEP pep,PetscInt nev,PetscInt *ncv,Petsc
   PetscInt       dim;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompareAny((PetscObject)pep,&krylov,PEPTOAR,PEPQARNOLDI,"");CHKERRQ(ierr);
-  dim = krylov?(pep->nmat-1)*pep->n:pep->n;
+  ierr = PetscObjectTypeCompareAny((PetscObject)pep,&krylov,PEPTOAR,PEPSTOAR,PEPQARNOLDI,"");CHKERRQ(ierr);
+  dim = (pep->nmat-1)*pep->n;
   if (*ncv) { /* ncv set */
     if (krylov) {
       if (*ncv<nev+1 && !(*ncv==nev && *ncv==dim)) SETERRQ(PetscObjectComm((PetscObject)pep),1,"The value of ncv must be at least nev+1");
@@ -437,7 +444,7 @@ PetscErrorCode PEPSetDimensions_Default(PEP pep,PetscInt nev,PetscInt *ncv,Petsc
    PEPAllocateSolution - Allocate memory storage for common variables such
    as eigenvalues and eigenvectors.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep   - eigensolver context
@@ -445,7 +452,7 @@ PetscErrorCode PEPSetDimensions_Default(PEP pep,PetscInt nev,PetscInt *ncv,Petsc
            working basis slightly larger than ncv
 
    Developers Note:
-   This is PETSC_EXTERN because it may be required by user plugin PEP
+   This is SLEPC_EXTERN because it may be required by user plugin PEP
    implementations.
 
    Level: developer
