@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,7 +17,7 @@
 /*@C
    PEPView - Prints the PEP data structure.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep - the polynomial eigenproblem solver context
@@ -38,8 +38,6 @@
    PetscViewerASCIIOpen() - output to a specified file.
 
    Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
 @*/
 PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
 {
@@ -52,7 +50,9 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)pep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)pep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(pep,1,viewer,2);
 
@@ -73,6 +73,7 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
       switch (pep->problem_type) {
         case PEP_GENERAL:    type = "general polynomial eigenvalue problem"; break;
         case PEP_HERMITIAN:  type = HERM " polynomial eigenvalue problem"; break;
+        case PEP_HYPERBOLIC: type = "hyperbolic polynomial eigenvalue problem"; break;
         case PEP_GYROSCOPIC: type = "gyroscopic polynomial eigenvalue problem"; break;
       }
     } else type = "not yet set";
@@ -127,6 +128,13 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
       case PEP_SMALLEST_IMAGINARY:
         ierr = PetscViewerASCIIPrintf(viewer,"smallest imaginary parts\n");CHKERRQ(ierr);
         break;
+      case PEP_ALL:
+        if (pep->inta || pep->intb) {
+          ierr = PetscViewerASCIIPrintf(viewer,"all eigenvalues in interval [%g,%g]\n",(double)pep->inta,(double)pep->intb);CHKERRQ(ierr);
+        } else {
+          ierr = PetscViewerASCIIPrintf(viewer,"all eigenvalues in the region\n");CHKERRQ(ierr);
+        }
+        break;
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",pep->nev);CHKERRQ(ierr);
@@ -186,6 +194,7 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
   if (!pep->st) { ierr = PEPGetST(pep,&pep->st);CHKERRQ(ierr); }
   ierr = STView(pep->st,viewer);CHKERRQ(ierr);
   if (pep->refine!=PEP_REFINE_NONE) {
+    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     if (pep->npart>1) {
       if (pep->refinesubc->color==0) {
         ierr = PetscViewerASCIIGetStdout(PetscSubcommChild(pep->refinesubc),&sviewer);CHKERRQ(ierr);
@@ -194,16 +203,41 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
     } else {
       ierr = KSPView(pep->refineksp,viewer);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   PEPViewFromOptions - View from options
+
+   Collective on PEP
+
+   Input Parameters:
++  pep  - the eigensolver context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: PEPView(), PEPCreate()
+@*/
+PetscErrorCode PEPViewFromOptions(PEP pep,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)pep,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PEPReasonView - Displays the reason a PEP solve converged or diverged.
 
-   Collective on PEP
+   Collective on pep
 
-   Parameter:
+   Input Parameters:
 +  pep - the eigensolver context
 -  viewer - the viewer to display the reason
 
@@ -237,9 +271,9 @@ PetscErrorCode PEPReasonView(PEP pep,PetscViewer viewer)
    PEPReasonViewFromOptions - Processes command line options to determine if/how
    the PEP converged reason is to be viewed.
 
-   Collective on PEP
+   Collective on pep
 
-   Input Parameters:
+   Input Parameter:
 .  pep - the eigensolver context
 
    Level: developer
@@ -255,7 +289,7 @@ PetscErrorCode PEPReasonViewFromOptions(PEP pep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPReasonView(pep,viewer);CHKERRQ(ierr);
@@ -289,7 +323,7 @@ static PetscErrorCode PEPErrorView_ASCII(PEP pep,PEPErrorType etype,PetscViewer
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
     for (j=0;j<PetscMin(8,pep->nev-8*i);j++) {
       k = pep->perm[8*i+j];
-      ierr = SlepcPrintEigenvalueASCII(pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
+      ierr = SlepcPrintEigenvalueASCII(viewer,pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
       if (8*i+j+1<pep->nev) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
@@ -362,7 +396,7 @@ static PetscErrorCode PEPErrorView_MATLAB(PEP pep,PEPErrorType etype,PetscViewer
    PEPErrorView - Displays the errors associated with the computed solution
    (as well as the eigenvalues).
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep    - the eigensolver context
@@ -392,7 +426,9 @@ PetscErrorCode PEPErrorView(PEP pep,PEPErrorType etype,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)pep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)pep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(pep,1,viewer,2);
   PEPCheckSolved(pep,1);
@@ -421,9 +457,9 @@ PetscErrorCode PEPErrorView(PEP pep,PEPErrorType etype,PetscViewer viewer)
    PEPErrorViewFromOptions - Processes command line options to determine if/how
    the errors of the computed solution are to be viewed.
 
-   Collective on PEP
+   Collective on pep
 
-   Input Parameters:
+   Input Parameter:
 .  pep - the eigensolver context
 
    Level: developer
@@ -439,21 +475,21 @@ PetscErrorCode PEPErrorViewFromOptions(PEP pep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_error_absolute",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_error_absolute",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPErrorView(pep,PEP_ERROR_ABSOLUTE,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_error_relative",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_error_relative",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPErrorView(pep,PEP_ERROR_RELATIVE,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_error_backward",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_error_backward",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPErrorView(pep,PEP_ERROR_BACKWARD,viewer);CHKERRQ(ierr);
@@ -504,7 +540,7 @@ static PetscErrorCode PEPValuesView_ASCII(PEP pep,PetscViewer viewer)
   for (i=0;i<pep->nconv;i++) {
     k = pep->perm[i];
     ierr = PetscViewerASCIIPrintf(viewer,"   ");CHKERRQ(ierr);
-    ierr = SlepcPrintEigenvalueASCII(pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
+    ierr = SlepcPrintEigenvalueASCII(viewer,pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
@@ -543,7 +579,7 @@ static PetscErrorCode PEPValuesView_MATLAB(PEP pep,PetscViewer viewer)
 /*@C
    PEPValuesView - Displays the computed eigenvalues in a viewer.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep    - the eigensolver context
@@ -564,7 +600,9 @@ PetscErrorCode PEPValuesView(PEP pep,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)pep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)pep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(pep,1,viewer,2);
   PEPCheckSolved(pep,1);
@@ -594,9 +632,9 @@ PetscErrorCode PEPValuesView(PEP pep,PetscViewer viewer)
    PEPValuesViewFromOptions - Processes command line options to determine if/how
    the computed eigenvalues are to be viewed.
 
-   Collective on PEP
+   Collective on pep
 
-   Input Parameters:
+   Input Parameter:
 .  pep - the eigensolver context
 
    Level: developer
@@ -612,7 +650,7 @@ PetscErrorCode PEPValuesViewFromOptions(PEP pep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_view_values",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_view_values",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPValuesView(pep,viewer);CHKERRQ(ierr);
@@ -626,9 +664,9 @@ PetscErrorCode PEPValuesViewFromOptions(PEP pep)
 /*@C
    PEPVectorsView - Outputs computed eigenvectors to a viewer.
 
-   Collective on PEP
+   Collective on pep
 
-   Parameter:
+   Input Parameters:
 +  pep    - the eigensolver context
 -  viewer - the viewer
 
@@ -655,7 +693,9 @@ PetscErrorCode PEPVectorsView(PEP pep,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)pep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)pep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(pep,1,viewer,2);
   PEPCheckSolved(pep,1);
@@ -678,9 +718,9 @@ PetscErrorCode PEPVectorsView(PEP pep,PetscViewer viewer)
    PEPVectorsViewFromOptions - Processes command line options to determine if/how
    the computed eigenvectors are to be viewed.
 
-   Collective on PEP
+   Collective on pep
 
-   Input Parameters:
+   Input Parameter:
 .  pep - the eigensolver context
 
    Level: developer
@@ -696,7 +736,7 @@ PetscErrorCode PEPVectorsViewFromOptions(PEP pep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,"-pep_view_vectors",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_view_vectors",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PEPVectorsView(pep,viewer);CHKERRQ(ierr);
