@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -208,47 +208,32 @@ PetscErrorCode BVNorm_Local_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
 PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
-  BV_MAT         *v = (BV_MAT*)V->data,*w = (BV_MAT*)W->data;
-  PetscScalar    *pv,*pw,*pb,*pc;
-  PetscInt       j,m;
+  PetscInt       j;
   PetscBool      flg;
+  Mat            Vmat,Wmat;
+  Vec            vv,ww;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(v->A,&pv);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(w->A,&pw);CHKERRQ(ierr);
   ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
   if (V->vmm && flg) {
-    m = V->k-V->l;
-    if (V->vmm==BV_MATMULT_MAT_SAVE) {
-      ierr = BV_AllocateMatMult(V,A,m);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(V->B,&pb);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pb,pv+(V->nc+V->l)*V->n,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(V->B,&pb);CHKERRQ(ierr);
-    } else {  /* BV_MATMULT_MAT */
-      ierr = MatCreateDense(PetscObjectComm((PetscObject)V),V->n,PETSC_DECIDE,V->N,m,pv+(V->nc+V->l)*V->n,&V->B);CHKERRQ(ierr);
-    }
-    if (!V->C) {
-      ierr = MatMatMultSymbolic(A,V->B,PETSC_DEFAULT,&V->C);CHKERRQ(ierr);
-    }
-    ierr = MatMatMultNumeric(A,V->B,V->C);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(V->C,&pc);CHKERRQ(ierr);
-    ierr = PetscMemcpy(pw+(W->nc+W->l)*W->n,pc,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(V->C,&pc);CHKERRQ(ierr);
-    if (V->vmm==BV_MATMULT_MAT) {
-      ierr = MatDestroy(&V->B);CHKERRQ(ierr);
-      ierr = MatDestroy(&V->C);CHKERRQ(ierr);
-    }
+    ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
+    ierr = MatProductCreateWithMat(A,Vmat,NULL,Wmat);CHKERRQ(ierr);
+    ierr = MatProductSetType(Wmat,MATPRODUCT_AB);CHKERRQ(ierr);
+    ierr = MatProductSetFromOptions(Wmat);CHKERRQ(ierr);
+    ierr = MatProductSymbolic(Wmat);CHKERRQ(ierr);
+    ierr = MatProductNumeric(Wmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
   } else {
     for (j=0;j<V->k-V->l;j++) {
-      ierr = VecPlaceArray(V->cv[1],pv+(V->nc+V->l+j)*V->n);CHKERRQ(ierr);
-      ierr = VecPlaceArray(W->cv[1],pw+(W->nc+W->l+j)*W->n);CHKERRQ(ierr);
-      ierr = MatMult(A,V->cv[1],W->cv[1]);CHKERRQ(ierr);
-      ierr = VecResetArray(V->cv[1]);CHKERRQ(ierr);
-      ierr = VecResetArray(W->cv[1]);CHKERRQ(ierr);
+      ierr = BVGetColumn(V,V->l+j,&vv);CHKERRQ(ierr);
+      ierr = BVGetColumn(W,W->l+j,&ww);CHKERRQ(ierr);
+      ierr = MatMult(A,vv,ww);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(V,V->l+j,&vv);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(W,W->l+j,&ww);CHKERRQ(ierr);
     }
   }
-  ierr = MatDenseRestoreArray(v->A,&pv);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(w->A,&pw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -263,38 +248,62 @@ PetscErrorCode BVCopy_Mat(BV V,BV W)
   ierr = MatDenseGetArray(w->A,&pw);CHKERRQ(ierr);
   pvc = pv+(V->nc+V->l)*V->n;
   pwc = pw+(W->nc+W->l)*W->n;
-  ierr = PetscMemcpy(pwc,pvc,(V->k-V->l)*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(pwc,pvc,(V->k-V->l)*V->n);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(v->A,&pv);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(w->A,&pw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode BVCopyColumn_Mat(BV V,PetscInt j,PetscInt i)
+{
+  PetscErrorCode ierr;
+  BV_MAT         *v = (BV_MAT*)V->data;
+  PetscScalar    *pv;
+
+  PetscFunctionBegin;
+  ierr = MatDenseGetArray(v->A,&pv);CHKERRQ(ierr);
+  ierr = PetscArraycpy(pv+(V->nc+i)*V->n,pv+(V->nc+j)*V->n,V->n);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(v->A,&pv);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode BVResize_Mat(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
   BV_MAT         *ctx = (BV_MAT*)bv->data;
   PetscScalar    *pA,*pnew;
+  PetscInt       lsplit;
   Mat            A;
   char           str[50];
+  PetscScalar    *array;
+  BV             parent;
 
   PetscFunctionBegin;
-  ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),bv->n,PETSC_DECIDE,PETSC_DECIDE,m,NULL,&A);CHKERRQ(ierr);
-  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)A);CHKERRQ(ierr);
-  if (((PetscObject)bv)->name) {
-    ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
-    ierr = PetscObjectSetName((PetscObject)A,str);CHKERRQ(ierr);
+  if (bv->issplit==2) {
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    ierr = MatDenseGetArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
+    ierr = MatDensePlaceArray(ctx->A,array+lsplit*bv->n);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
+  } else if (!bv->issplit) {
+    ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),bv->n,PETSC_DECIDE,PETSC_DECIDE,m,NULL,&A);CHKERRQ(ierr);
+    ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+    ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)A);CHKERRQ(ierr);
+    if (((PetscObject)bv)->name) {
+      ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
+      ierr = PetscObjectSetName((PetscObject)A,str);CHKERRQ(ierr);
+    }
+    if (copy) {
+      ierr = MatDenseGetArray(ctx->A,&pA);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(A,&pnew);CHKERRQ(ierr);
+      ierr = PetscArraycpy(pnew,pA,PetscMin(m,bv->m)*bv->n);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(ctx->A,&pA);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(A,&pnew);CHKERRQ(ierr);
+    }
+    ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
+    ctx->A = A;
   }
-  if (copy) {
-    ierr = MatDenseGetArray(ctx->A,&pA);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(A,&pnew);CHKERRQ(ierr);
-    ierr = PetscMemcpy(pnew,pA,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(ctx->A,&pA);CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(A,&pnew);CHKERRQ(ierr);
-  }
-  ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
-  ctx->A = A;
   PetscFunctionReturn(0);
 }
 
@@ -375,10 +384,11 @@ PetscErrorCode BVView_Mat(BV bv,PetscViewer viewer)
   const char        *bvname,*name;
 
   PetscFunctionBegin;
-  ierr = MatView(ctx->A,viewer);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) PetscFunctionReturn(0);
+    ierr = MatView(ctx->A,viewer);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_MATLAB) {
       ierr = PetscObjectGetName((PetscObject)bv,&bvname);CHKERRQ(ierr);
       ierr = PetscObjectGetName((PetscObject)ctx->A,&name);CHKERRQ(ierr);
@@ -387,6 +397,8 @@ PetscErrorCode BVView_Mat(BV bv,PetscViewer viewer)
         ierr = PetscViewerASCIIPrintf(viewer,"%s=%s(:,%D:end);\n",bvname,bvname,bv->nc+1);CHKERRQ(ierr);
       }
     }
+  } else {
+    ierr = MatView(ctx->A,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -404,13 +416,15 @@ PetscErrorCode BVDestroy_Mat(BV bv)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
+SLEPC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
 {
   PetscErrorCode ierr;
   BV_MAT         *ctx;
-  PetscInt       nloc,bs;
+  PetscInt       nloc,bs,lsplit;
   PetscBool      seq;
   char           str[50];
+  PetscScalar    *array,*ptr;
+  BV             parent;
 
   PetscFunctionBegin;
   ierr = PetscNewLog(bv,&ctx);CHKERRQ(ierr);
@@ -425,7 +439,18 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   ierr = VecGetLocalSize(bv->t,&nloc);CHKERRQ(ierr);
   ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
 
-  ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),nloc,PETSC_DECIDE,PETSC_DECIDE,bv->m,NULL,&ctx->A);CHKERRQ(ierr);
+  if (bv->issplit) {
+    /* split BV: share the memory of the parent BV */
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    ierr = MatDenseGetArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
+    ptr = (bv->issplit==1)? array: array+lsplit*nloc;
+    ierr = MatDenseRestoreArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
+  } else {
+    /* regular BV: allocate memory for the BV entries */
+    ptr = NULL;
+  }
+  ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),nloc,PETSC_DECIDE,PETSC_DECIDE,bv->m,ptr,&ctx->A);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(ctx->A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(ctx->A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)ctx->A);CHKERRQ(ierr);
@@ -439,13 +464,8 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
     ierr = MatDestroy(&bv->Acreate);CHKERRQ(ierr);
   }
 
-  if (ctx->mpi) {
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[0]);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[1]);CHKERRQ(ierr);
-  } else {
-    ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,NULL,&bv->cv[0]);CHKERRQ(ierr);
-    ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,NULL,&bv->cv[1]);CHKERRQ(ierr);
-  }
+  ierr = VecDuplicateEmpty(bv->t,&bv->cv[0]);CHKERRQ(ierr);
+  ierr = VecDuplicateEmpty(bv->t,&bv->cv[1]);CHKERRQ(ierr);
 
   bv->ops->mult             = BVMult_Mat;
   bv->ops->multvec          = BVMultVec_Mat;
@@ -459,6 +479,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   bv->ops->norm_local       = BVNorm_Local_Mat;
   bv->ops->matmult          = BVMatMult_Mat;
   bv->ops->copy             = BVCopy_Mat;
+  bv->ops->copycolumn       = BVCopyColumn_Mat;
   bv->ops->resize           = BVResize_Mat;
   bv->ops->getcolumn        = BVGetColumn_Mat;
   bv->ops->restorecolumn    = BVRestoreColumn_Mat;
