@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -295,11 +295,25 @@ PetscErrorCode BVMatMult_Vecs(BV V,Mat A,BV W)
   PetscErrorCode ierr;
   BV_VECS        *v = (BV_VECS*)V->data,*w = (BV_VECS*)W->data;
   PetscInt       j;
+  PetscBool      flg;
+  Mat            Vmat,Wmat;
 
   PetscFunctionBegin;
-  if (V->vmm) { ierr = PetscInfo(V,"BVMatMult_Vecs: ignoring method\n");CHKERRQ(ierr); }
-  for (j=0;j<V->k-V->l;j++) {
-    ierr = MatMult(A,v->V[V->nc+V->l+j],w->V[W->nc+W->l+j]);CHKERRQ(ierr);
+  ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
+  if (V->vmm && flg) {
+    ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
+    ierr = MatProductCreateWithMat(A,Vmat,NULL,Wmat);CHKERRQ(ierr);
+    ierr = MatProductSetType(Wmat,MATPRODUCT_AB);CHKERRQ(ierr);
+    ierr = MatProductSetFromOptions(Wmat);CHKERRQ(ierr);
+    ierr = MatProductSymbolic(Wmat);CHKERRQ(ierr);
+    ierr = MatProductNumeric(Wmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
+  } else {
+    for (j=0;j<V->k-V->l;j++) {
+      ierr = MatMult(A,v->V[V->nc+V->l+j],w->V[W->nc+W->l+j]);CHKERRQ(ierr);
+    }
   }
   PetscFunctionReturn(0);
 }
@@ -317,30 +331,47 @@ PetscErrorCode BVCopy_Vecs(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode BVCopyColumn_Vecs(BV V,PetscInt j,PetscInt i)
+{
+  PetscErrorCode ierr;
+  BV_VECS        *v = (BV_VECS*)V->data;
+
+  PetscFunctionBegin;
+  ierr = VecCopy(v->V[V->nc+j],v->V[V->nc+i]);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode BVResize_Vecs(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx = (BV_VECS*)bv->data;
   Vec            *newV;
-  PetscInt       j;
+  PetscInt       j,lsplit;
   char           str[50];
+  BV             parent;
 
   PetscFunctionBegin;
-  ierr = VecDuplicateVecs(bv->t,m,&newV);CHKERRQ(ierr);
-  ierr = PetscLogObjectParents(bv,m,newV);CHKERRQ(ierr);
-  if (((PetscObject)bv)->name) {
-    for (j=0;j<m;j++) {
-      ierr = PetscSNPrintf(str,50,"%s_%D",((PetscObject)bv)->name,j);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)newV[j],str);CHKERRQ(ierr);
+  if (bv->issplit==2) {
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    ctx->V = ((BV_VECS*)parent->data)->V+lsplit;
+  } else if (!bv->issplit) {
+    ierr = VecDuplicateVecs(bv->t,m,&newV);CHKERRQ(ierr);
+    ierr = PetscLogObjectParents(bv,m,newV);CHKERRQ(ierr);
+    if (((PetscObject)bv)->name) {
+      for (j=0;j<m;j++) {
+        ierr = PetscSNPrintf(str,50,"%s_%D",((PetscObject)bv)->name,j);CHKERRQ(ierr);
+        ierr = PetscObjectSetName((PetscObject)newV[j],str);CHKERRQ(ierr);
+      }
     }
-  }
-  if (copy) {
-    for (j=0;j<PetscMin(m,bv->m);j++) {
-      ierr = VecCopy(ctx->V[j],newV[j]);CHKERRQ(ierr);
+    if (copy) {
+      for (j=0;j<PetscMin(m,bv->m);j++) {
+        ierr = VecCopy(ctx->V[j],newV[j]);CHKERRQ(ierr);
+      }
     }
+    ierr = VecDestroyVecs(bv->m,&ctx->V);CHKERRQ(ierr);
+    ctx->V = newV;
   }
-  ierr = VecDestroyVecs(bv->m,&ctx->V);CHKERRQ(ierr);
-  ctx->V = newV;
   PetscFunctionReturn(0);
 }
 
@@ -366,7 +397,7 @@ PetscErrorCode BVGetArray_Vecs(BV bv,PetscScalar **a)
   ierr = PetscMalloc1((bv->nc+bv->m)*bv->n,a);CHKERRQ(ierr);
   for (j=0;j<bv->nc+bv->m;j++) {
     ierr = VecGetArrayRead(ctx->V[j],&p);CHKERRQ(ierr);
-    ierr = PetscMemcpy(*a+j*bv->n,p,bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(*a+j*bv->n,p,bv->n);CHKERRQ(ierr);
     ierr = VecRestoreArrayRead(ctx->V[j],&p);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -382,7 +413,7 @@ PetscErrorCode BVRestoreArray_Vecs(BV bv,PetscScalar **a)
   PetscFunctionBegin;
   for (j=0;j<bv->nc+bv->m;j++) {
     ierr = VecGetArray(ctx->V[j],&p);CHKERRQ(ierr);
-    ierr = PetscMemcpy(p,*a+j*bv->n,bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(p,*a+j*bv->n,bv->n);CHKERRQ(ierr);
     ierr = VecRestoreArray(ctx->V[j],&p);CHKERRQ(ierr);
   }
   ierr = PetscFree(*a);CHKERRQ(ierr);
@@ -397,10 +428,10 @@ PetscErrorCode BVGetArrayRead_Vecs(BV bv,const PetscScalar **a)
   const PetscScalar *p;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc1((bv->nc+bv->m)*bv->n,a);CHKERRQ(ierr);
+  ierr = PetscMalloc1((bv->nc+bv->m)*bv->n,(PetscScalar**)a);CHKERRQ(ierr);
   for (j=0;j<bv->nc+bv->m;j++) {
     ierr = VecGetArrayRead(ctx->V[j],&p);CHKERRQ(ierr);
-    ierr = PetscMemcpy((PetscScalar*)*a+j*bv->n,p,bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy((PetscScalar*)*a+j*bv->n,p,bv->n);CHKERRQ(ierr);
     ierr = VecRestoreArrayRead(ctx->V[j],&p);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -438,8 +469,7 @@ PetscErrorCode BVSetFromOptions_Vecs(PetscOptionItems *PetscOptionsObject,BV bv)
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"BV Vecs Options");CHKERRQ(ierr);
 
-    ierr = PetscOptionsInt("-bv_vecs_vmip","Version of BVMultInPlace operation","",ctx->vmip,&ctx->vmip,NULL);CHKERRQ(ierr);
-    if (ctx->vmip<0 || ctx->vmip>1) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_OUTOFRANGE,"Wrong version of BVMultInPlace");
+    ierr = PetscOptionsRangeInt("-bv_vecs_vmip","Version of BVMultInPlace operation","",ctx->vmip,&ctx->vmip,NULL,0,1);CHKERRQ(ierr);
     ierr = BVVecsSetVmip(bv,ctx->vmip);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
@@ -459,6 +489,7 @@ PetscErrorCode BVView_Vecs(BV bv,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) PetscFunctionReturn(0);
     if (format == PETSC_VIEWER_ASCII_MATLAB) ismatlab = PETSC_TRUE;
   }
   if (ismatlab) {
@@ -481,39 +512,50 @@ PetscErrorCode BVDestroy_Vecs(BV bv)
   BV_VECS        *ctx = (BV_VECS*)bv->data;
 
   PetscFunctionBegin;
-  ierr = VecDestroyVecs(bv->nc+bv->m,&ctx->V);CHKERRQ(ierr);
+  if (!bv->issplit) { ierr = VecDestroyVecs(bv->nc+bv->m,&ctx->V);CHKERRQ(ierr); }
   ierr = PetscFree(bv->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode BVDuplicate_Vecs(BV V,BV *W)
+PetscErrorCode BVDuplicate_Vecs(BV V,BV W)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx = (BV_VECS*)V->data;
 
   PetscFunctionBegin;
-  ierr = BVVecsSetVmip(*W,ctx->vmip);CHKERRQ(ierr);
+  ierr = BVVecsSetVmip(W,ctx->vmip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
+SLEPC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx;
-  PetscInt       j;
+  PetscInt       j,lsplit;
   PetscBool      isgpu;
   char           str[50];
+  BV             parent;
+  Vec            *Vpar;
 
   PetscFunctionBegin;
   ierr = PetscNewLog(bv,&ctx);CHKERRQ(ierr);
   bv->data = (void*)ctx;
 
-  ierr = VecDuplicateVecs(bv->t,bv->m,&ctx->V);CHKERRQ(ierr);
-  ierr = PetscLogObjectParents(bv,bv->m,ctx->V);CHKERRQ(ierr);
-  if (((PetscObject)bv)->name) {
-    for (j=0;j<bv->m;j++) {
-      ierr = PetscSNPrintf(str,50,"%s_%D",((PetscObject)bv)->name,j);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)ctx->V[j],str);CHKERRQ(ierr);
+  if (bv->issplit) {
+    /* split BV: share the Vecs of the parent BV */
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    Vpar   = ((BV_VECS*)parent->data)->V;
+    ctx->V = (bv->issplit==1)? Vpar: Vpar+lsplit;
+  } else {
+    /* regular BV: create array of Vecs to store the BV columns */
+    ierr = VecDuplicateVecs(bv->t,bv->m,&ctx->V);CHKERRQ(ierr);
+    ierr = PetscLogObjectParents(bv,bv->m,ctx->V);CHKERRQ(ierr);
+    if (((PetscObject)bv)->name) {
+      for (j=0;j<bv->m;j++) {
+        ierr = PetscSNPrintf(str,50,"%s_%D",((PetscObject)bv)->name,j);CHKERRQ(ierr);
+        ierr = PetscObjectSetName((PetscObject)ctx->V[j],str);CHKERRQ(ierr);
+      }
     }
   }
 
@@ -525,9 +567,12 @@ PETSC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
   }
 
   /* Default version of BVMultInPlace */
-  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&isgpu,VECSEQCUDA,VECMPICUDA,VECSEQCUSP,VECMPICUSP,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&isgpu,VECSEQCUDA,VECMPICUDA,"");CHKERRQ(ierr);
   ctx->vmip = isgpu? 1: 0;
 
+  /* Default BVMatMult method */
+  bv->vmm = BV_MATMULT_VECS;
+
   /* Deferred call to setfromoptions */
   if (bv->defersfo) {
     ierr = PetscObjectOptionsBegin((PetscObject)bv);CHKERRQ(ierr);
@@ -549,6 +594,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
   bv->ops->norm_end         = BVNorm_End_Vecs;
   bv->ops->matmult          = BVMatMult_Vecs;
   bv->ops->copy             = BVCopy_Vecs;
+  bv->ops->copycolumn       = BVCopyColumn_Vecs;
   bv->ops->resize           = BVResize_Vecs;
   bv->ops->getcolumn        = BVGetColumn_Vecs;
   bv->ops->getarray         = BVGetArray_Vecs;
