@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,7 +17,7 @@
 /*@
    BVMult - Computes Y = beta*Y + alpha*X*Q.
 
-   Logically Collective on BV
+   Logically Collective on Y
 
    Input Parameters:
 +  Y,X        - basis vectors
@@ -61,6 +61,7 @@ PetscErrorCode BVMult(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
   if (Q) PetscValidHeaderSpecific(Q,MAT_CLASSID,5);
   PetscValidType(Y,1);
   BVCheckSizes(Y,1);
+  BVCheckOp(Y,1,mult);
   PetscValidType(X,4);
   BVCheckSizes(X,4);
   if (Q) PetscValidType(Q,5);
@@ -85,7 +86,7 @@ PetscErrorCode BVMult(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 /*@
    BVMultVec - Computes y = beta*y + alpha*X*q.
 
-   Logically Collective on BV and Vec
+   Logically Collective on X
 
    Input Parameters:
 +  X          - a basis vectors object
@@ -122,9 +123,10 @@ PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScal
   PetscValidLogicalCollectiveScalar(X,alpha,2);
   PetscValidLogicalCollectiveScalar(X,beta,3);
   PetscValidHeaderSpecific(y,VEC_CLASSID,4);
-  PetscValidPointer(q,5);
+  PetscValidScalarPointer(q,5);
   PetscValidType(X,1);
   BVCheckSizes(X,1);
+  BVCheckOp(X,1,multvec);
   PetscValidType(y,4);
   PetscCheckSameComm(X,1,y,4);
 
@@ -142,7 +144,7 @@ PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScal
    BVMultColumn - Computes y = beta*y + alpha*X*q, where y is the j-th column
    of X.
 
-   Logically Collective on BV
+   Logically Collective on X
 
    Input Parameters:
 +  X          - a basis vectors object
@@ -199,7 +201,7 @@ PetscErrorCode BVMultColumn(BV X,PetscScalar alpha,PetscScalar beta,PetscInt j,P
 /*@
    BVMultInPlace - Update a set of vectors as V(:,s:e-1) = V*Q(:,s:e-1).
 
-   Logically Collective on BV
+   Logically Collective on V
 
    Input Parameters:
 +  Q - a sequential dense matrix
@@ -256,7 +258,7 @@ PetscErrorCode BVMultInPlace(BV V,Mat Q,PetscInt s,PetscInt e)
 /*@
    BVMultInPlaceTranspose - Update a set of vectors as V(:,s:e-1) = V*Q'(:,s:e-1).
 
-   Logically Collective on BV
+   Logically Collective on V
 
    Input Parameters:
 +  Q - a sequential dense matrix
@@ -308,7 +310,7 @@ PetscErrorCode BVMultInPlaceTranspose(BV V,Mat Q,PetscInt s,PetscInt e)
 /*@
    BVScale - Multiply the BV entries by a scalar value.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv    - basis vectors
@@ -344,7 +346,7 @@ PetscErrorCode BVScale(BV bv,PetscScalar alpha)
 /*@
    BVScaleColumn - Scale one column of a BV.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv    - basis vectors
@@ -405,7 +407,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVSetRandomColumn_Private(BV bv,PetscInt k)
 /*@
    BVSetRandom - Set the columns of a BV to random numbers.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 .  bv - basis vectors
@@ -440,7 +442,7 @@ PetscErrorCode BVSetRandom(BV bv)
 /*@
    BVSetRandomColumn - Set one column of a BV to random numbers.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv - basis vectors
@@ -473,7 +475,7 @@ PetscErrorCode BVSetRandomColumn(BV bv,PetscInt j)
    BVSetRandomCond - Set the columns of a BV to random numbers, in a way that
    the generated matrix has a given condition number.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv    - basis vectors
@@ -492,7 +494,7 @@ PetscErrorCode BVSetRandomCond(BV bv,PetscReal condn)
   PetscInt       k,i;
   PetscScalar    *eig,*d;
   DS             ds;
-  Mat            A,X,Y,Xt,M;
+  Mat            A,X,Xt,M,G;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
@@ -523,19 +525,21 @@ PetscErrorCode BVSetRandomCond(BV bv,PetscReal condn)
   ierr = MatDenseGetArray(M,&d);CHKERRQ(ierr);
   for (i=0;i<bv->k;i++) d[i+i*bv->m] = (1.0/condn+(1.0-1.0/condn)/(bv->k-1)*i)/PetscSqrtScalar(eig[i]);
   ierr = MatDenseRestoreArray(M,&d);CHKERRQ(ierr);
-  /* M = X*M*X' */
-  ierr = MatCreateSeqDense(PETSC_COMM_SELF,bv->k,bv->k,NULL,&Y);CHKERRQ(ierr);
+  /* G = X*M*X' */
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,bv->k,bv->k,NULL,&Xt);CHKERRQ(ierr);
   ierr = DSGetMat(ds,DS_MAT_X,&X);CHKERRQ(ierr);
-  ierr = MatMatMult(X,M,MAT_REUSE_MATRIX,PETSC_DEFAULT,&Y);CHKERRQ(ierr);
   ierr = MatTranspose(X,MAT_REUSE_MATRIX,&Xt);CHKERRQ(ierr);
-  ierr = MatMatMult(Y,Xt,MAT_REUSE_MATRIX,PETSC_DEFAULT,&M);CHKERRQ(ierr);
+  ierr = MatProductCreate(Xt,M,NULL,&G);CHKERRQ(ierr);
+  ierr = MatProductSetType(G,MATPRODUCT_PtAP);CHKERRQ(ierr);
+  ierr = MatProductSetFromOptions(G);CHKERRQ(ierr);
+  ierr = MatProductSymbolic(G);CHKERRQ(ierr);
+  ierr = MatProductNumeric(G);CHKERRQ(ierr);
   ierr = MatDestroy(&X);CHKERRQ(ierr);
-  /* B = B*M */
-  ierr = BVMultInPlace(bv,M,bv->l,bv->k);CHKERRQ(ierr);
-  ierr = MatDestroy(&Y);CHKERRQ(ierr);
   ierr = MatDestroy(&Xt);CHKERRQ(ierr);
   ierr = MatDestroy(&M);CHKERRQ(ierr);
+  /* B = B*G */
+  ierr = BVMultInPlace(bv,G,bv->l,bv->k);CHKERRQ(ierr);
+  ierr = MatDestroy(&G);CHKERRQ(ierr);
   ierr = PetscFree(eig);CHKERRQ(ierr);
   ierr = DSDestroy(&ds);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
@@ -546,7 +550,7 @@ PetscErrorCode BVSetRandomCond(BV bv,PetscReal condn)
 /*@
    BVMatMult - Computes the matrix-vector product for each column, Y=A*V.
 
-   Neighbor-wise Collective on Mat and BV
+   Neighbor-wise Collective on A
 
    Input Parameters:
 +  V - basis vectors context
@@ -555,17 +559,19 @@ PetscErrorCode BVSetRandomCond(BV bv,PetscReal condn)
    Output Parameter:
 .  Y - the result
 
-   Note:
+   Notes:
    Both V and Y must be distributed in the same manner. Only active columns
    (excluding the leading ones) are processed.
    In the result Y, columns are overwritten starting from the leading ones.
+   The number of active columns in V and Y should match, although they need
+   not be the same columns.
 
    It is possible to choose whether the computation is done column by column
    or as a Mat-Mat product, see BVSetMatMultMethod().
 
    Level: beginner
 
-.seealso: BVCopy(), BVSetActiveColumns(), BVMatMultColumn(), BVMatMultHermitianTranspose(), BVSetMatMultMethod()
+.seealso: BVCopy(), BVSetActiveColumns(), BVMatMultColumn(), BVMatMultTranspose(), BVMatMultHermitianTranspose(), BVSetMatMultMethod()
 @*/
 PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
 {
@@ -575,6 +581,7 @@ PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
   PetscValidHeaderSpecific(V,BV_CLASSID,1);
   PetscValidType(V,1);
   BVCheckSizes(V,1);
+  BVCheckOp(V,1,matmult);
   PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   PetscValidType(A,2);
   PetscValidHeaderSpecific(Y,BV_CLASSID,3);
@@ -583,7 +590,7 @@ PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
   PetscCheckSameComm(V,1,A,2);
   PetscCheckSameTypeAndComm(V,1,Y,3);
   if (V->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
-  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
+  if (V->k-V->l!=Y->k-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D active columns, should match %D active columns in V",Y->k-Y->l,V->k-V->l);
 
   ierr = PetscLogEventBegin(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
   ierr = (*V->ops->matmult)(V,A,Y);CHKERRQ(ierr);
@@ -593,10 +600,10 @@ PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
 }
 
 /*@
-   BVMatMultHermitianTranspose - Computes the matrix-vector product with the
-   conjugate transpose of a matrix for each column, Y=A^H*V.
+   BVMatMultTranspose - Computes the matrix-vector product with the transpose
+   of a matrix for each column, Y=A^T*V.
 
-   Neighbor-wise Collective on Mat and BV
+   Neighbor-wise Collective on A
 
    Input Parameters:
 +  V - basis vectors context
@@ -605,23 +612,23 @@ PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
    Output Parameter:
 .  Y - the result
 
-   Note:
+   Notes:
    Both V and Y must be distributed in the same manner. Only active columns
    (excluding the leading ones) are processed.
    In the result Y, columns are overwritten starting from the leading ones.
+   The number of active columns in V and Y should match, although they need
+   not be the same columns.
 
-   As opposed to BVMatMult(), this operation is always done column by column,
-   with a sequence of calls to MatMultHermitianTranspose().
+   Currently implemented via MatCreateTranspose().
 
    Level: beginner
 
-.seealso: BVCopy(), BVSetActiveColumns(), BVMatMult(), BVMatMultColumn()
+.seealso: BVMatMult(), BVMatMultHermitianTranspose()
 @*/
-PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
+PetscErrorCode BVMatMultTranspose(BV V,Mat A,BV Y)
 {
   PetscErrorCode ierr;
-  PetscInt       j;
-  Vec            z,f;
+  Mat            AT;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(V,BV_CLASSID,1);
@@ -637,16 +644,60 @@ PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
   if (V->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
   if (V->k-V->l>Y->m-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
 
-  ierr = PetscLogEventBegin(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
-  for (j=0;j<V->k-V->l;j++) {
-    ierr = BVGetColumn(V,V->l+j,&z);CHKERRQ(ierr);
-    ierr = BVGetColumn(Y,Y->l+j,&f);CHKERRQ(ierr);
-    ierr = MatMultHermitianTranspose(A,z,f);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(V,V->l+j,&z);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(Y,Y->l+j,&f);CHKERRQ(ierr);
-  }
-  ierr = PetscLogEventEnd(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
-  ierr = PetscObjectStateIncrease((PetscObject)Y);CHKERRQ(ierr);
+  ierr = MatCreateTranspose(A,&AT);CHKERRQ(ierr);
+  ierr = BVMatMult(V,AT,Y);CHKERRQ(ierr);
+  ierr = MatDestroy(&AT);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVMatMultHermitianTranspose - Computes the matrix-vector product with the
+   conjugate transpose of a matrix for each column, Y=A^H*V.
+
+   Neighbor-wise Collective on A
+
+   Input Parameters:
++  V - basis vectors context
+-  A - the matrix
+
+   Output Parameter:
+.  Y - the result
+
+   Note:
+   Both V and Y must be distributed in the same manner. Only active columns
+   (excluding the leading ones) are processed.
+   In the result Y, columns are overwritten starting from the leading ones.
+   The number of active columns in V and Y should match, although they need
+   not be the same columns.
+
+   Currently implemented via MatCreateHermitianTranspose().
+
+   Level: beginner
+
+.seealso: BVMatMult(), BVMatMultTranspose()
+@*/
+PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
+{
+  PetscErrorCode ierr;
+  Mat            AH;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(V,BV_CLASSID,1);
+  PetscValidType(V,1);
+  BVCheckSizes(V,1);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  PetscValidType(A,2);
+  PetscValidHeaderSpecific(Y,BV_CLASSID,3);
+  PetscValidType(Y,3);
+  BVCheckSizes(Y,3);
+  PetscCheckSameComm(V,1,A,2);
+  PetscCheckSameTypeAndComm(V,1,Y,3);
+  if (V->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
+  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
+
+  ierr = MatCreateHermitianTranspose(A,&AH);CHKERRQ(ierr);
+  ierr = BVMatMult(V,AH,Y);CHKERRQ(ierr);
+  ierr = MatDestroy(&AH);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -654,19 +705,16 @@ PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
    BVMatMultColumn - Computes the matrix-vector product for a specified
    column, storing the result in the next column: v_{j+1}=A*v_j.
 
-   Neighbor-wise Collective on Mat and BV
+   Neighbor-wise Collective on A
 
    Input Parameters:
 +  V - basis vectors context
 .  A - the matrix
 -  j - the column
 
-   Output Parameter:
-.  Y - the result
-
    Level: beginner
 
-.seealso: BVMatMult()
+.seealso: BVMatMult(), BVMatMultTransposeColumn(), BVMatMultHermitianTransposeColumn()
 @*/
 PetscErrorCode BVMatMultColumn(BV V,Mat A,PetscInt j)
 {
@@ -679,6 +727,7 @@ PetscErrorCode BVMatMultColumn(BV V,Mat A,PetscInt j)
   BVCheckSizes(V,1);
   PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   PetscCheckSameComm(V,1,A,2);
+  PetscValidLogicalCollectiveInt(V,j,3);
   if (j<0) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Index j must be non-negative");
   if (j+1>=V->m) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Result should go in index j+1=%D but BV only has %D columns",j+1,V->m);
 
@@ -693,3 +742,85 @@ PetscErrorCode BVMatMultColumn(BV V,Mat A,PetscInt j)
   PetscFunctionReturn(0);
 }
 
+/*@
+   BVMatMultTransposeColumn - Computes the transpose matrix-vector product for a
+   specified column, storing the result in the next column: v_{j+1}=A^T*v_j.
+
+   Neighbor-wise Collective on A
+
+   Input Parameters:
++  V - basis vectors context
+.  A - the matrix
+-  j - the column
+
+   Level: beginner
+
+.seealso: BVMatMult(), BVMatMultColumn(), BVMatMultHermitianTransposeColumn()
+@*/
+PetscErrorCode BVMatMultTransposeColumn(BV V,Mat A,PetscInt j)
+{
+  PetscErrorCode ierr;
+  Vec            vj,vj1;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(V,BV_CLASSID,1);
+  PetscValidType(V,1);
+  BVCheckSizes(V,1);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  PetscCheckSameComm(V,1,A,2);
+  PetscValidLogicalCollectiveInt(V,j,3);
+  if (j<0) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Index j must be non-negative");
+  if (j+1>=V->m) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Result should go in index j+1=%D but BV only has %D columns",j+1,V->m);
+
+  ierr = PetscLogEventBegin(BV_MatMultVec,V,A,0,0);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,j,&vj);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,j+1,&vj1);CHKERRQ(ierr);
+  ierr = MatMultTranspose(A,vj,vj1);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,j+1,&vj1);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(BV_MatMultVec,V,A,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)V);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVMatMultHermitianTransposeColumn - Computes the matrix-vector product for a specified
+   column, storing the result in the next column: v_{j+1}=A*v_j.
+
+   Neighbor-wise Collective on A
+
+   Input Parameters:
++  V - basis vectors context
+.  A - the matrix
+-  j - the column
+
+   Level: beginner
+
+.seealso: BVMatMult(), BVMatMultColumn(), BVMatMultTransposeColumn()
+@*/
+PetscErrorCode BVMatMultHermitianTransposeColumn(BV V,Mat A,PetscInt j)
+{
+  PetscErrorCode ierr;
+  Vec            vj,vj1;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(V,BV_CLASSID,1);
+  PetscValidType(V,1);
+  BVCheckSizes(V,1);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  PetscCheckSameComm(V,1,A,2);
+  PetscValidLogicalCollectiveInt(V,j,3);
+  if (j<0) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Index j must be non-negative");
+  if (j+1>=V->m) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Result should go in index j+1=%D but BV only has %D columns",j+1,V->m);
+
+  ierr = PetscLogEventBegin(BV_MatMultVec,V,A,0,0);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,j,&vj);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,j+1,&vj1);CHKERRQ(ierr);
+  ierr = MatMultHermitianTranspose(A,vj,vj1);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,j+1,&vj1);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(BV_MatMultVec,V,A,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)V);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
