@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,17 +17,22 @@
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscinitialize_              PETSCINITIALIZE
 #define petscinitializenoarguments_   PETSCINITIALIZENOARGUMENTS
+#define petscfinalize_                PETSCFINALIZE
 #define slepcinitialize_              SLEPCINITIALIZE
 #define slepcinitializenoarguments_   SLEPCINITIALIZENOARGUMENTS
+#define slepcfinalize_                SLEPCFINALIZE
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define petscinitialize_              petscinitialize
 #define petscinitializenoarguments_   petscinitializenoarguments
+#define petscfinalize_                petscfinalize
 #define slepcinitialize_              slepcinitialize
 #define slepcinitializenoarguments_   slepcinitializenoarguments
+#define slepcfinalize_                slepcfinalize
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscinitialize_(char *filename PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len));
-PETSC_EXTERN void PETSC_STDCALL petscinitializenoarguments_(PetscErrorCode *ierr);
+SLEPC_EXTERN void petscinitialize_(char *filename,PetscErrorCode *ierr,PETSC_FORTRAN_CHARLEN_T len);
+SLEPC_EXTERN void petscinitializenoarguments_(PetscErrorCode *ierr);
+SLEPC_EXTERN void petscfinalize_(PetscErrorCode *ierr);
 
 /*
     SlepcInitialize - Version called from Fortran.
@@ -74,13 +79,25 @@ static void slepcinitialize_internal(char *filename,PetscInt len,PetscBool argum
   if (*ierr) { (*PetscErrorPrintf)("SlepcInitialize:Calling PetscInfo()");return; }
 }
 
-PETSC_EXTERN void PETSC_STDCALL slepcinitialize_(char *filename PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+SLEPC_EXTERN void slepcinitialize_(char *filename,PetscErrorCode *ierr,PETSC_FORTRAN_CHARLEN_T len)
 {
   slepcinitialize_internal(filename,len,PETSC_TRUE,ierr);
 }
 
-PETSC_EXTERN void PETSC_STDCALL slepcinitializenoarguments_(PetscErrorCode *ierr)
+SLEPC_EXTERN void slepcinitializenoarguments_(PetscErrorCode *ierr)
 {
   slepcinitialize_internal(NULL,(PetscInt)0,PETSC_FALSE,ierr);
 }
 
+SLEPC_EXTERN void slepcfinalize_(PetscErrorCode *ierr)
+{
+  *ierr = PetscInfo(0,"SlepcFinalize called from Fortran\n");
+  if (*ierr) { (*PetscErrorPrintf)("SlepcFinalize:Calling PetscInfo()");return; }
+  *ierr = 0;
+  if (SlepcBeganPetsc) {
+    petscfinalize_(ierr);
+    if (*ierr) { (*PetscErrorPrintf)("SlepcFinalize:Calling petscfinalize_()");return; }
+  }
+  SlepcInitializeCalled = PETSC_FALSE;
+}
+
