@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -10,6 +10,12 @@
 
 #include <slepc/private/slepcimpl.h>           /*I "slepcsys.h" I*/
 
+#if defined(SLEPC_HAVE_HPDDM)
+#include <petscksp.h>
+SLEPC_EXTERN PetscErrorCode KSPCreate_HPDDM(KSP);
+SLEPC_EXTERN PetscErrorCode PCCreate_HPDDM(PC);
+#endif
+
 /*@C
     SlepcGetVersion - Gets the SLEPc version information in a string.
 
@@ -74,7 +80,7 @@ PetscErrorCode SlepcGetVersionNumber(PetscInt *major,PetscInt *minor,PetscInt *s
 /*
    SlepcPrintVersion - Prints SLEPc version info.
 
-   Collective on MPI_Comm
+   Collective
 */
 static PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 {
@@ -94,7 +100,7 @@ static PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 /*
    SlepcPrintHelpIntro - Prints introductory SLEPc help info.
 
-   Collective on MPI_Comm
+   Collective
 */
 static PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 {
@@ -133,6 +139,20 @@ static PetscErrorCode SlepcLoadDynamicLibrary(const char *name,PetscBool *found)
 }
 #endif
 
+#if defined(PETSC_HAVE_THREADSAFETY)
+SLEPC_EXTERN PetscErrorCode STInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode DSInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode FNInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode BVInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode RGInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode EPSInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode SVDInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode PEPInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode NEPInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode MFNInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode LMEInitializePackage(void);
+#endif
+
 /*
     SlepcInitialize_DynamicLibraries - Adds the default dynamic link libraries to the
     search path.
@@ -186,6 +206,11 @@ PetscErrorCode SlepcInitialize_DynamicLibraries(void)
   ierr = MFNInitializePackage();CHKERRQ(ierr);
   ierr = LMEInitializePackage();CHKERRQ(ierr);
 #endif
+
+#if defined(SLEPC_HAVE_HPDDM)
+  ierr = KSPRegister(KSPHPDDM,KSPCreate_HPDDM);CHKERRQ(ierr);
+  ierr = PCRegister(PCHPDDM,PCCreate_HPDDM);CHKERRQ(ierr);
+#endif
   PetscFunctionReturn(0);
 }
 
@@ -207,9 +232,9 @@ PetscErrorCode SlepcCitationsInitialize()
   ierr = PetscCitationsRegister("@TechReport{slepc-manual,\n"
     "   author = \"J. E. Roman and C. Campos and E. Romero and A. Tomas\",\n"
     "   title = \"{SLEPc} Users Manual\",\n"
-    "   number = \"DSIC-II/24/02 - Revision 3.9\",\n"
+    "   number = \"DSIC-II/24/02 - Revision 3.13\",\n"
     "   institution = \"D. Sistemes Inform\\`atics i Computaci\\'o, Universitat Polit\\`ecnica de Val\\`encia\",\n"
-    "   year = \"2018\"\n"
+    "   year = \"2020\"\n"
     "}\n",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -318,7 +343,7 @@ PetscErrorCode SlepcInitializeNoArguments(void)
 PetscErrorCode SlepcInitialized(PetscBool *isInitialized)
 {
   PetscFunctionBegin;
-  PetscValidPointer(isInitialized,1);
+  PetscValidBoolPointer(isInitialized,1);
   *isInitialized = SlepcInitializeCalled;
   PetscFunctionReturn(0);
 }
